@@ -1,3 +1,4 @@
+import asyncio
 import time
 from ..Scheduler import IntervalTask
 from ..dbserver.base import Base, ENGINE_DICT, mysql
@@ -19,6 +20,8 @@ class data_manager(object):
         self.min_keep_connection = db_config['min_keep_connection'] if 'min_keep_connection' in db_config else 1
         if 'engine' not in self.t_data or self.t_data['engine'] not in ENGINE_DICT:
             self.t_data['engine'] = mysql
+        if 'charset' not in self.t_data:
+            self.t_data['charset'] = 'utf8mb4'
 
         self.sql_pool = {}
         print('creating DB connection pool...')
@@ -26,11 +29,8 @@ class data_manager(object):
         self.new()
         print('connect done!')
         IntervalTask(data_manager.keep_cycle, self.keep_connect)
-
         self.mem_cache = {}
-    # def kill_hanged_connection(self):
-    #     dead_sql = []
-    #
+
     def get_table_name(self,table_name):
         if '.' in table_name:
             return table_name
@@ -38,36 +38,30 @@ class data_manager(object):
             return self.t_data['database']+'.'+table_name
 
     def keep_connect(self):
+        # print('查看死亡链接')
         can_delete_sql = []
         # can_update_sql = []
         count = 0
         for sql in self.sql_pool.values():
-            if int(time.time()) - sql.last_execute_time > 30:
+            if int(time.time()) - sql.last_execute_time > 40:
                 count += 1
-                # if count > self.min_keep_connection:
                 can_delete_sql.append(id(sql))
-                # else:
-                #     can_update_sql.append(id(sql))
-        self.kill_unused_connection(can_delete_sql)
 
-        # 保持心跳
-        # for sql_id in can_update_sql:
-        #     sql = self.sql_pool[sql_id]
-        #     if sql.is_busy() == False:
-        #         sql.become_busy()
-        #         sql.keep_connect()
-        #         sql.become_free()
-        # 保持心跳
+        self.kill_unused_connection(can_delete_sql)
+        for sql in self.sql_pool.values():
+            if sql.is_busy() == False:
+                sql.become_busy()
+                sql.keep_connect()
+                sql.become_free()
+
         return
 
     def kill_unused_connection(self, kill_connect):
         count = 0
         for sql_id in kill_connect:
             if sql_id in self.sql_pool:
-                # if self.sql_pool[sql_id].is_busy() == False:
                 self.sql_pool.pop(sql_id)
                 count += 1
-        # print('计划清理',len(kill_connect),'个空闲连接，实际清理',count,'个')
         return
 
     def get_table_data(self,table,query):
@@ -83,11 +77,16 @@ class data_manager(object):
             return None
 
     def find_free_sql(self):
-        for sql in self.sql_pool.values():
-            if sql.is_busy() is False:
-                sql.become_busy()
-                return sql
-        # print('数据库连接池全忙状态，创建新的数据库链接')
+        try:
+            sql_list = self.sql_pool.values()
+            for sql in sql_list:
+                if sql.is_busy() is False and int(time.time()) - sql.last_execute_time < 29:
+                    sql.become_busy()
+                    return sql
+        except Exception as e:
+            print(str(e),'查找空闲sql链接时遍历错误')
+            # pass
+        print('数据库连接池全忙状态，创建新的数据库链接')
         sql = self.create_new_sql()
         sql.become_busy()
         self.add_new_sql(sql)
@@ -106,7 +105,7 @@ class data_manager(object):
         return
 
     def create_new_sql(self, ):
-        sql = Base(self.t_data['host'], self.t_data['user'], self.t_data['password'], self.t_data['database'],self.t_data['engine'])
+        sql = Base(self.t_data['host'], self.t_data['user'], self.t_data['password'], self.t_data['database'],self.t_data['engine'],self.t_data['charset'])
         # sql.become_busy()
         # self.sql_pool.append(sql)
         return sql
@@ -133,8 +132,9 @@ class data_manager(object):
         sql = self.find_free_sql()
         table = self.get_table_name(table)
         if from_cache == True:
-            query = sql.find_info(table, conditions, or_cond, fields, None, order, None)
+            query,params = sql.find_info(table, conditions, or_cond, fields, None, order, None)
             query += " limit 1"
+            query = query%params
             res = self.get_table_data(table,query)
             if res is not None:
                 sql.become_free()
@@ -154,7 +154,8 @@ class data_manager(object):
         # sql.become_busy()
         # print('执行这次sql请求的链接是', id(sql))
         if from_cache == True:
-            query = sql.find_info(table, conditions, or_cond, fields, group, order, limit)
+            query,params = sql.find_info(table, conditions, or_cond, fields, group, order, limit)
+            query = query % params
             res = self.get_table_data(table, query)
             if res is not None:
                 sql.become_free()
